#include "shiftDialog.h"

#include "plugin.hpp"
#include "farManager.h"
#include "ssaTime.h"
#include "helpContexts.h"

#include <algorithm>


const char* ShiftDialog::label0_;
const char* ShiftDialog::label1_;
const char* ShiftDialog::label2_;
const char* ShiftDialog::help_;
LONGLONG ShiftDialog::init1_;
LONGLONG ShiftDialog::init2_;
LONGLONG ShiftDialog::mediaTime_;
LONGLONG ShiftDialog::shift_;

ShiftDialog::ShiftDialog(const char* label0,
		const char* label1, LONGLONG init1,
		const char* label2, LONGLONG init2,
		LONGLONG mediaTime,
		const char* help)
{
	label0_ = label0;
	label1_ = label1;
	label2_ = label2;
	help_ = help;
	init1_ = init1;
	init2_ = init2;
	mediaTime_ = mediaTime;
	shift_ = 0;
}

bool ShiftDialog::execute()
{
	FarDialogItem items[] =
	{
		{ DI_TEXT,     3, 1, 12, 0, 0, 0,              0, 0, "Shift &by"   },
		{ DI_FIXEDIT, 15, 1, 25, 0, 1, 0,   DIF_MASKEDIT, 1, " 0:00:00.00" },
		{ DI_TEXT,     3, 2, 12, 0, 0, 0,              0, 0, "&Start time" },
		{ DI_FIXEDIT, 16, 2, 25, 0, 0, 0,   DIF_MASKEDIT, 0, ""            },
		{ DI_TEXT,     3, 3, 12, 0, 0, 0,              0, 0, "&End time"   },
		{ DI_FIXEDIT, 16, 3, 25, 0, 0, 0,   DIF_MASKEDIT, 0, ""            },
		{ DI_BUTTON,  29, 1, 40, 0, 0, 0, DIF_BTNNOCLOSE, 0, "&Original"   },
		{ DI_BUTTON,  29, 2, 40, 0, 0, 0, DIF_BTNNOCLOSE, 0, "&Current "   }
	};

	strncpy(items[0].Data, label0_, COUNTOF(items[0].Data));
	strncpy(items[2].Data, label1_, COUNTOF(items[2].Data));
	strncpy(items[4].Data, label2_, COUNTOF(items[4].Data));

	items[1].Mask = "#9:99:99.99";
	items[3].Mask = items[5].Mask = "9:99:99.99";

	formatTime(init1_, items[3].Data, items[3].Data + 10);
	formatTime(init2_, items[5].Data, items[5].Data + 10);

	if (mediaTime_ < 0)
	{
		items[7].Flags |= DIF_DISABLE;
	}

	return -1 != farManager.dialogEx(items, COUNTOF(items), 43, 5, help_, &dialogProc);
}

void ShiftDialog::shiftChanged(HANDLE hDlg, const FarDialogItem& item)
{
	shift_ = parseTime(item.Data + 1, item.Data + 11);
	if ('-' == item.Data[0]) shift_ = -shift_;

	char buf[11];

	formatTime((std::max)(0LL, shift_ + init1_), buf, buf + 10); buf[10] = 0;
	farManager.setTextPtr(hDlg, 3, buf);

	formatTime((std::max)(0LL, shift_ + init2_), buf, buf + 10);
	farManager.setTextPtr(hDlg, 5, buf);
}

void ShiftDialog::time1Changed(HANDLE hDlg, const FarDialogItem& item)
{
	LONGLONG time1 = parseTime(item.Data, item.Data + 10);
	shift_ = time1 - init1_;

	char buf[12];

	buf[0] = shift_ < 0 ? '-' : ' ';
	formatTime(_abs64(shift_), buf + 1, buf + 11);
	buf[11] = 0;
	farManager.setTextPtr(hDlg, 1, buf);

	formatTime((std::max)(0LL, shift_ + init2_), buf, buf + 10);
	buf[10] = 0;
	farManager.setTextPtr(hDlg, 5, buf);
}
void ShiftDialog::time2Changed(HANDLE hDlg, const FarDialogItem& item)
{
	LONGLONG time2 = parseTime(item.Data, item.Data + 10);
	shift_ = time2 - init2_;

	char buf[12];

	buf[0] = shift_ < 0 ? '-' : ' ';
	formatTime(_abs64(shift_), buf + 1, buf + 11);
	buf[11] = 0;
	farManager.setTextPtr(hDlg, 1, buf);

	formatTime((std::max)(0LL, shift_ + init1_), buf, buf + 10);
	buf[10] = 0;
	farManager.setTextPtr(hDlg, 3, buf);
}

void ShiftDialog::originalClick(HANDLE hDlg)
{
	shift_ = 0LL;
	farManager.setTextPtr(hDlg, 1, " 0:00:00.00");

	char buf[11];

	formatTime(init1_, buf, buf + 10);
	buf[10] = 0;
	farManager.setTextPtr(hDlg, 3, buf);

	formatTime(init2_, buf, buf + 10);
	farManager.setTextPtr(hDlg, 5, buf);
}

void ShiftDialog::currentClick(HANDLE hDlg)
{
	shift_ = mediaTime_ - init1_;

	char buf[12];

	buf[0] = shift_ < 0 ? '-' : ' ';
	formatTime(_abs64(shift_), buf + 1, buf + 11);
	buf[11] = 0;
	farManager.setTextPtr(hDlg, 1, buf);

	formatTime((std::max)(0LL, shift_ + init1_), buf, buf + 10);
	buf[10] = 0;
	farManager.setTextPtr(hDlg, 3, buf);

	formatTime((std::max)(0LL, shift_ + init2_), buf, buf + 10);
	buf[10] = 0;
	farManager.setTextPtr(hDlg, 5, buf);
}

long WINAPI ShiftDialog::dialogProc(HANDLE hDlg, int Msg, int Param1, long Param2)
{
	switch (Msg)
	{
	case DN_EDITCHANGE:
		switch (Param1)
		{
		case 1:
			shiftChanged(hDlg, *reinterpret_cast<FarDialogItem*>(Param2));
			break;
		case 3:
			time1Changed(hDlg, *reinterpret_cast<FarDialogItem*>(Param2));
			break;
		case 5:
			time2Changed(hDlg, *reinterpret_cast<FarDialogItem*>(Param2));
			break;
		}
		break;
	case DN_BTNCLICK:
		switch (Param1)
		{
		case 6:
			originalClick(hDlg);
			break;
		case 7:
			currentClick(hDlg);
			break;
		}
		break;
	}
	return farManager.defDlgProc(hDlg, Msg, Param1, Param2);
}
